#!/bin/bash
set -e
CI_COMMIT_REF_NAME=$1
IMAGE=$2
SERVICE_SHORT_NAME=$3
CI_COMMIT_TAG=$4

echo "service: $SERVICE_SHORT_NAME"
echo "commit_ref: $CI_COMMIT_REF_NAME"
echo "image: $IMAGE"
echo "tag: $CI_COMMIT_TAG"

# tags
if [[ $CI_COMMIT_TAG != "" ]];
then
    echo "Detected tag"
    docker pull $IMAGE:latest || true
    docker build --cache-from $IMAGE:latest -t "$IMAGE":dev -t $IMAGE:$CI_COMMIT_REF_NAME $SERVICE_SHORT_NAME/ || exit 1;
    docker push $IMAGE:$CI_COMMIT_REF_NAME

# master/staging
elif [[ $CI_COMMIT_REF_NAME = 'master' ]] || [[ $CI_COMMIT_REF_NAME = 'staging' ]];
then
    echo "Detected master/staging"
    if [[ "$CI_COMMIT_REF_NAME" = "master" ]] ; then TAG_USED="latest"; else TAG_USED="staging"; fi
    docker pull "$IMAGE":"$TAG_USED" || true
    docker build --cache-from "$IMAGE":"$TAG_USED" -t "$IMAGE":dev -t "$IMAGE":"$TAG_USED" $SERVICE_SHORT_NAME/ || exit 1;
    docker push "$IMAGE":"$TAG_USED"

# branches
else
    echo "Detected branch"
    docker pull "$IMAGE":staging || true
    docker build --cache-from "$IMAGE":staging -t "$IMAGE":dev $SERVICE_SHORT_NAME/ || exit 1;
    mkdir image
    docker save "$IMAGE":dev > image/$SERVICE_SHORT_NAME.tar
    gzip image/$SERVICE_SHORT_NAME.tar
fi
