// vsclient3d webpack 1 configuration including cesium 1.62 related changes
const webpack = require('webpack');
const path = require('path');
const CopywebpackPlugin = require('copy-webpack-plugin');

const packageJson = require('./package.json');
const eoxcPackageJson = require('eoxc/package.json');

const cesiumWorkers = '../Build/Cesium/Workers';
const cesiumSource = 'node_modules/cesium/Source';

module.exports = {
  entry: {
    vsclient: ['babel-polyfill', './src/preload.js', './src/main.js'],
  },
  resolve: {
    modulesDirectories: ['web_modules', 'node_modules', 'bower_components'],
    alias: {
      // necessary to avoid multiple packings of backbone due to marionette
      backbone: path.join(__dirname, 'node_modules', 'backbone', 'backbone'),
      cesium: path.join(__dirname, cesiumSource),
      handlebars: 'handlebars/dist/handlebars.min.js',
      'opensearch-browser': 'opensearch-browser/dist',
      jquery: path.join(__dirname, 'node_modules/jquery/src/jquery'),
      almond: path.join(__dirname,'node_modules/cesium-navigation/bower_components/almond/almond'),
      KnockoutES5: path.join(__dirname,'node_modules/cesium-navigation/bower_components/knockout-es5/dist/knockout-es5.min'),
      knockout: path.join(__dirname,'node_modules/cesium-navigation/bower_components/knockout/dist/knockout'),
      Hammer: path.join(__dirname,'node_modules/cesium-navigation/bower_components/hammerjs/hammer.min'),
      'markdownIt': path.join(__dirname,'node_modules/cesium-navigation/bower_components/markdown-it/dist/markdown-it.min'),
      'markdownItSanitizer': path.join(__dirname,'node_modules/cesium-navigation/bower_components/markdown-it-sanitizer/dist/markdown-it-sanitizer.min'),
    },
  },
  resolveLoader: {
    root: path.join(__dirname, 'node_modules'),
  },
  output: {
    path: path.join(__dirname, 'dist'),
    filename: '[name].js',
    library: 'vsclient',
    libraryTarget: 'umd',
    // Needed to compile multiline strings in Cesium
    sourcePrefix: '',
  },
  node: {
    // Resolve node module use of fs
    fs: 'empty',
},
  devServer: {
    host: '0.0.0.0',
    inline: true,
    disableHostCheck: true,
  },
  module: {
    loaders: [
      { test: /node_modules.*eoxc.*js$/, loader: 'babel-loader' },
      { test: /node_modules.*opensearch.*js$/, loader: 'babel-loader' },
      { test: /node_modules.*ol.*js$/, loader: 'babel-loader' },
      { test: /\.js$/, exclude: /node_modules/, loader: 'babel-loader' },
      { test: /\.json$/, loader: 'json-loader' },
      { test: /\.coffee$/, loader: 'coffee-loader' },
      { test: /\.litcoffee$/, loader: 'coffee-loader?literate' },
      { test: /\.css$/, loaders: ['style', 'css', 'postcss'] },
      { test: /\.less$/, loaders: ['style', 'css', 'postcss', 'less'] },
      { test: /\.scss$/, loaders: ['style', 'css', 'postcss', 'sass'] },
      { test: /\.(gif|jpg|jpeg)$/, loader: 'file-loader'},
      { test: /\.hbs$/, loader: 'handlebars-loader?helperDirs[]=' + __dirname + '/src/helpers' },
      // for anything that might be included in a css
      { test: /\.(png|woff|woff2|eot|ttf|otf|svg)(\?v=[0-9]\.[0-9]\.[0-9])?$/, loader: 'url-loader?limit=4000000' },
    ],
  },
  plugins: [
    new webpack.ResolverPlugin(
        new webpack.ResolverPlugin.DirectoryDescriptionFilePlugin('.bower.json', ['main'])
    ),
    new webpack.optimize.DedupePlugin(),
    new webpack.ProvidePlugin({
      $: 'jquery',
      jQuery: 'jquery',
      'window.jQuery': 'jquery',
      Promise: 'bluebird'
    }),
    new webpack.BannerPlugin(
      `VS Client version: ${packageJson.version}\neoxc version: ${eoxcPackageJson.version}`
    ),
    new CopywebpackPlugin([
          { from: path.join(cesiumSource, cesiumWorkers), to: 'Workers' },
          { from: path.join(cesiumSource, 'Assets'), to: 'Assets' },
          { from: path.join(cesiumSource, 'Widgets'), to: 'Widgets' },
      ]),
    new webpack.DefinePlugin({
      // Define relative base path in cesium for loading assets
      CESIUM_BASE_URL: JSON.stringify('')
    }),
  ],
  postcss() {
    return [
      require('autoprefixer'),
    ];
  },
  devtool: 'source-map',
  cache: true,
};
