# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
# import os
# import sys
# sys.path.insert(0, os.path.abspath('.'))


# -- Project information -----------------------------------------------------

project = u'View Server - Operator Guide'
copyright = u'2019, 2020, EOX IT Services GmbH'
author = u'EOX IT Services GmbH'

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx_multiversion",
    "sphinx.ext.intersphinx",
    "sphinx.ext.autodoc",
    "sphinx.ext.mathjax",
    "sphinx.ext.viewcode",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'
html_title = project
master_doc = 'index'


# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []

html_sidebars = {
    '**': [
        'versions.html',
    ],
}


html_favicon = 'images/favicon.ico'

numfig = True
numfig_format = {
    'figure': 'Figure %s',
    'table': 'Table %s',
    'code-block': 'Listing %s',
    'section': 'Section',
}
numfig_secnum_depth = 2

latex_documents = [(
    'index',
    'View-Server_-_Operator-Guide.tex',
    u'View Server - Operator Guide',
    u'EOX IT Services GmbH',
    'manual',
    True
)]
latex_elements = {
    'papersize': 'a4paper',
}

import os
import re
import collections
from sphinx_multiversion.git import get_all_refs

def get_latest_from_each_minor(refs):

    Version = collections.namedtuple(
        "Version",
        ["major", "minor", "patch"]
    )

    tags = []
    for ref in refs:
        if ref.source == 'tags' and re.match('^release-.*', ref.name):
            release = ref.name.strip("release-").split("-")[0]
            major, minor, patch = release.split(".") 
            tags.append(Version(major, minor, patch))

    tags = sorted(tags)

    eligible_tags = []
    for tag in tags:
        if not eligible_tags:
            eligible_tags.append(tag)
            continue
        
        last_tag = eligible_tags[-1]
        if tag.major == last_tag.major and tag.minor == last_tag.minor and tag.patch > last_tag.patch:
            del eligible_tags[-1]
            eligible_tags.append(tag)
        else:
            eligible_tags.append(tag)

    eligible_tags = eligible_tags[-3:]

    return_regex = ""
    for tag in eligible_tags:
        return_regex += f"^release-{tag.major}.{tag.minor}.{tag.patch}|"

    return return_regex[:-1]

branch_name = os.environ.get('BRANCH_NAME')
project_dir = os.environ.get('CI_PROJECT_DIR')

refs = get_all_refs(project_dir)

branch_regex = r'^master|^staging' 
smv_branch_whitelist = branch_regex if not branch_name else branch_regex + f"|^{branch_name}"
smv_tag_whitelist = get_latest_from_each_minor(refs)
