import os
import re
import datetime
from typing import TextIO

import yaml


ENV_PATTERN = re.compile(r'.*?\${(\w+)}.*?')


def constructor_env_variables(loader, node):
        """
        Extracts the environment variable from the node's value
        :param yaml.Loader loader: the yaml loader
        :param node: the current node in the yaml
        :return: the parsed string that contains the value of the environment
        variable
        """
        value = loader.construct_scalar(node)
        match = ENV_PATTERN.findall(value)  # to find all env variables in line
        if match:
            full_value = value
            for g in match:
                env_variable = os.environ.get(g, )
                if env_variable is not None:
                    full_value = full_value.replace(
                        f'${{{g}}}', env_variable
                    )
                else:
                    return None
            return full_value
        return value

def now(loader, node):
    return datetime.datetime.now(tz=datetime.timezone.utc).isoformat()

tags = {
    '!env': constructor_env_variables,
    '!now': now,
}


def load_config(input_file: TextIO) -> dict:
    loader = yaml.SafeLoader

    # the tag will be used to mark where to start searching for the pattern
    # e.g. somekey: !env somestring${MYENVVAR}blah blah blah
    for tag, func in tags.items():
        if tag == '!env':
            loader.add_implicit_resolver(tag, ENV_PATTERN, None)
        loader.add_constructor(tag, func)
        
    return yaml.load(input_file, Loader=loader)
