import logging

import requests

from .Endpoint import Endpoint

logger = logging.getLogger(__name__)

class STACAPIEndpoint(Endpoint):
    type = 'STACAPI'

    def __init__(self, *args, **kwargs):
        super(STACAPIEndpoint, self).__init__(*args, **kwargs)

    def harvest(self) -> list:
        # prepare query
        main = requests.get(self.url).json()
        search_url = next(link['href'] for link in main['links'] if link['rel'] == 'search')
        search_params = self.query.stacapi_params(main)
        result = []

        # query
        logger.debug('querying %s with %s', search_url, search_params)
        query_data = requests.get(search_url, params=search_params).json()
        context = query_data['context']
        while context['returned']:
            for feature in query_data['features']:
                result.append(feature)
            search_params['page'] += 1
            logger.debug('querying %s with %s', search_url, search_params)
            query_data = requests.get(search_url, params=search_params).json()
            context = query_data['context']

        # filter data  
        if self.filter:
            result = self.filter_data(result)

        
        return result

