from lxml import etree
from .util import pairwise


def evaluate_xpath(root, xpath):
    """
    """
    result = root.xpath(xpath, namespaces=root.nsmap)
    if result:
        if isinstance(result, list):
            return result[0]
        return result
    return None


def extract_product_types_and_levels(metadata_files, config):
    """
    """
    product_types = []
    product_levels = []
    for metadata_file in metadata_files:
        with open(metadata_file) as f:
            tree = etree.parse(f)
            root = tree.getroot()

        xpaths = config['type_extractor']['xpath']
        xpaths = [xpaths] if isinstance(xpaths, str) else xpaths
        for xpath in xpaths:
            product_type = evaluate_xpath(root, xpath)
            if product_type is not None:
                product_types.append(product_type)

        xpaths = config['level_extractor']['xpath']
        if xpaths:
            xpaths = [xpaths] if isinstance(xpaths, str) else xpaths
            for xpath in xpaths:
                product_level = evaluate_xpath(root, xpath)
                if product_level is not None:
                    product_levels.append(product_level)

    return product_types, product_levels


def parse_ring(string):
    raw_coords = string.split()
    return [(lon, lat) for lat, lon in pairwise(raw_coords)]


def serialize_coord_list(coords):
    return ','.join(
        f'{x} {y}' for x, y in coords
    )


def parse_polygons_gsc(elem):
    interior = serialize_coord_list(
        parse_ring(
            elem.xpath(
                "gml:exterior/gml:LinearRing/gml:posList", namespaces=elem.nsmap
            )[0].text.strip()
        )
    )

    exteriors = [
        f'''({
            serialize_coord_list(
                parse_ring(poslist_elem.text.strip())
            )
        })'''
        for poslist_elem in elem.xpath(
            "gml:interior/gml:LinearRing/gml:posList", namespaces=elem.nsmap
        )
    ]

    return f"POLYGON(({interior}){',' if exteriors else ''}{','.join(exteriors)})"


def extract_footprint(metadata_files, footprint_extractor: str = '//gml:target/eop:Footprint/gml:multiExtentOf/gml:MultiSurface/gml:surfaceMembers/gml:Polygon'):
    footprint = None
    for metadata_file in metadata_files:
        with open(metadata_file) as f:
            tree = etree.parse(f)
            root = tree.getroot()
        footprint = evaluate_xpath(root, footprint_extractor)
        if footprint is not None:
            break
    return parse_polygons_gsc(footprint)
