#!/bin/bash -e
echo "Running configure.sh" >&2

# substitute template values
cat /shibboleth2_template.xml \
    | sed -e "s/{{SPEntityID}}/$(echo ${SPEntityID} | sed -e 's/[]\/$*.^[]/\\&/g')/g" \
    | sed -e "s/{{IDPEntityID}}/$(echo ${IDPEntityID} | sed -e 's/[]\/$*.^[]/\\&/g')/g" \
     > /etc/shibboleth/shibboleth2.xml

# split apache access env var separated by | to list separated by spaces for access.xml
create_xml_user_category_rules () {
  IFS='|'
  xml_access=" <OR>"
  for usergroup in $1; do
    # either spField2 is equal to usergroup or spField3 contains "CDSLicense=usergroup"
    xml_access="${xml_access} <Rule list=\"false\" require=\"spField2\">${usergroup}</Rule>"
    xml_access="${xml_access} <RuleRegex require=\"spField3\">.*CDSLicense=${usergroup}.*</RuleRegex>"
  done
  xml_access="${xml_access} </OR>"
  echo $xml_access
}

USER_CATEGORY_ALLOW_LIST=$(create_xml_user_category_rules "$USER_CATEGORY_ALLOW_RENDERER") 
# renderer access rules template fill
cat /pass_accessrules_template.xml \
   | sed -e "s/{{USER_CATEGORY_ALLOW_LIST}}/$(echo ${USER_CATEGORY_ALLOW_LIST} | sed -e 's/[]\/$*.^[]/\\&/g')/g" \
    > /etc/shibboleth/pass-ac.xml

# cache access rules template fill
USER_CATEGORY_ALLOW_LIST=$(create_xml_user_category_rules "$USER_CATEGORY_ALLOW_CACHE") 
cat /pass_accessrules_template.xml \
  | sed -e "s/{{USER_CATEGORY_ALLOW_LIST}}/$(echo ${USER_CATEGORY_ALLOW_LIST} | sed -e 's/[]\/$*.^[]/\\&/g')/g" \
    > /etc/shibboleth/pass-ac-cache.xml
