#!/bin/sh
any_test_failed=1 # false
# fetch secrets and write them to their according files
cat $emg_db > ../config/emg/emg_db.env
cat $emg_django > ../config/emg/emg_django.env
cat $emg_obs > ../config/emg/emg_obs.env

# randomize the bucket name to avoid clash of two CI pipelines (linux)
PREPROCESSOR_RESULTS_BUCKET=$(cat /proc/sys/kernel/random/uuid)
echo "PREPROCESSOR_RESULTS_BUCKET=$PREPROCESSOR_RESULTS_BUCKET" >> ../config/emg/emg.env

# use `pvs_testing` bucket instead
sed -i -e 's/emg-data/pvs_testing/g' ../config/emg/emg.env
sed -i -e 's/emg-cache/pvs_testing/g' ../config/emg/emg_obs.env
# for CI update client config to use nginx path, instead of webpack dev-server one
sed -i -e 's/target: /index.html/target: /usr/share/nginx/html/index.html/g' ../docker-compose.emg.dev.yml

echo "DJANGO_SECRET_KEY=${DJANGO_SECRET_KEY}" >> ../config/emg/emg.env
# source the env file
set -o allexport
source ../config/emg/emg.env
set +o allexport

mkdir data
docker swarm init
docker network create -d overlay emg-extnet

# create the docker secrets
printf $OS_PASSWORD_DOWNLOAD | docker secret create OS_PASSWORD_DOWNLOAD -
printf $DJANGO_PASSWORD | docker secret create DJANGO_PASSWORD -
printf $DJANGO_SECRET_KEY | docker secret create DJANGO_SECRET_KEY -
printf $OS_PASSWORD | docker secret create OS_PASSWORD -

# create docker configs
printf $sftp_users_emg | docker config create sftp_users_emg -

# override preprocessor default configuration file
cat ./testing_preprocessor_config.yml>../config/emg/emg_preprocessor-config.yml

# deploy the stack
docker stack deploy -c ../docker-compose.emg.yml -c ../docker-compose.emg.dev.yml emg-pvs

./docker-stack-wait.sh -n renderer -n registrar -n preprocessor -n database -n sftp -n ingestor emg-pvs

# perform the testing
bash ./preprocessor_prep.sh preprocessed_list.csv
bash ./registrar_prep.sh product_list.csv

pytest registrar_test.py preprocessor_test.py renderer_test.py
if [ $? -ne 0 ]; then any_test_failed=0;fi

swift delete "$PREPROCESSOR_RESULTS_BUCKET"_segments
swift delete "$PREPROCESSOR_RESULTS_BUCKET"
# ingestor test
bash ./ingestor_prep.sh

if [ $? -ne 0 ]; then any_test_failed=0;fi
# cleanup
swift delete "$PREPROCESSOR_RESULTS_BUCKET"_segments
swift delete "$PREPROCESSOR_RESULTS_BUCKET"
for service in $(docker service ls --format "{{.Name}}"); do
  docker service ps $service --no-trunc >> "$CI_PROJECT_DIR/$service.txt"
  docker service logs $service --no-trunc >> "$CI_PROJECT_DIR/$service.txt" 2>&1
done

if [ $any_test_failed -ne 1 ]; then echo "Some test failed"; exit 1;fi
